/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Bookmark Previews extension.
 *
 * The Initial Developer of the Original Code is
 * John Marshall <JohnM555@gmail.com>.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   John Marshall <JohnM555@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var ThumbView = null;
var PlacesPreviewUtils = {
  utils : null,
  displayRoot : null,
  contentDeck : null,
  selected : null,
  dir : null,
  onPlaceSelected: function PO2_onPlaceSelected(resetSearchBox) {
    // Don't change the right-hand pane contents when there's no selection
    if (!this._places.hasSelection)
      return;

    var node = this._places.selectedNode;
    var queries = asQuery(node).getQueries({});

    // Items are only excluded on the left pane
    var options = node.queryOptions.clone();
    options.excludeItems = false;
    var placeURI = PlacesUtils.history.queriesToQueryString(queries, queries.length, options);
    //dump("place selected("+resetSearchBox+"): \n"+placeURI+"\n"+this._content.place+"\n");
    /*
     Stops the view from being refreshed when it shouldn't.
     Not the greatest fix, but right now I think it works
    */
    if (placeURI == this._content.place && resetSearchBox)
      return;
    // update the right-pane contents
    this._content.place = placeURI;

    // This just updates the back/forward buttons, it doesn't call us back
    // because node.uri is our current selection.
    this.location = node.uri;

    // Make sure the search UI is hidden.
    PlacesSearchBox.hideSearchUI();
    if (resetSearchBox) {
      var searchFilter = document.getElementById("searchFilter");
      searchFilter.reset();
    }

    // Update the "Find in <current collection>" command and the gray text in
    // the search box in the toolbar if the active collection is the current
    // collection.
    var findCommand = document.getElementById("OrganizerCommand_find:current");
    var findLabel = PlacesUtils.getFormattedString("findInPrefix", [node.title]);
    findCommand.setAttribute("label", findLabel);
    if (PlacesSearchBox.filterCollection == "collection")
      PlacesSearchBox.updateCollectionTitle(node.title);
  },

  init : function(){
    PlacesOrganizer.onPlaceSelected = PlacesPreviewUtils.onPlaceSelected;
    //dump("preview init\n");
    PlacesPreviewUtils.dir = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Components.interfaces.nsIProperties)
                     .get("ProfD", Components.interfaces.nsIFile);
    PlacesPreviewUtils.dir.append("bookmarkpreviews");
    if (!PlacesPreviewUtils.dir.exists())
      PlacesPreviewUtils.dir.create(Components.interfaces.nsIFile.DIRECTORY_TYPE,0777);

    ThumbView = document.getElementById("thumbnailView");

    PlacesOrganizer._content.addEventListener("select",PlacesPreviewUtils.onSelectionChanged,false);

    PlacesPreviewUtils.contentDeck = document.getElementById("contentDeck");
    var detailsMenu = document.getElementById("detailsViewMenu");
    if (detailsMenu.getAttribute("checked"))
      PlacesPreviewUtils.onSelectView('details');
    else{
      var thumbsMenu = detailsMenu.nextSibling;
      if (!thumbsMenu.getAttribute("checked"))
        PlacesPreviewUtils.onSelectView('details');
      else
        PlacesPreviewUtils.onSelectView('thumbnails');
    }
    var createPreviewsItem = document.getElementById("orgCreatePreviews");

    /* Move the info pane outside of the deck so it can be used by the thumbnail view*/
    var contentView = PlacesPreviewUtils.contentDeck.parentNode;
    var infoSplitter = document.getElementById("contentSplitter");
    var infoPane = document.getElementById("infoPane");
    infoSplitter = infoSplitter.parentNode.removeChild(infoSplitter);
    infoSplitter = contentView.appendChild(infoSplitter);
    infoPane = infoPane.parentNode.removeChild(infoPane);
    infoPane = contentView.appendChild(infoPane);
    addEventListener("resize",PlacesPreviewUtils.onResize,false);
    /* Listen for when the previews get updated */
    var observerService = Components.classes["@mozilla.org/observer-service;1"]
                          .getService(Components.interfaces.nsIObserverService);
    observerService.addObserver(PlacesPreviewUtils, "bookmarkpreviews:preview-updated", false);
  },
  uninit : function(){
    PlacesOrganizer._content.removeEventListener("select",PlacesPreviewUtils.onSelectionChanged,false);
    //dump("remove resize\n");
    window.removeEventListener("resize",PlacesPreviewUtils.onResize,false);
    var observerService = Components.classes["@mozilla.org/observer-service;1"]
                            .getService(Components.interfaces.nsIObserverService);
    observerService.removeObserver(PlacesPreviewUtils, "bookmarkpreviews:preview-updated");
    //dump("observer removed\n");
    if (ThumbView.initiated)
      ThumbView.uninit();
    if (AlbumView.initiated)
      AlbumView.uninit();
  },
  onResize : function(event){
    PlacesPreviewUtils.utils.onResize(event);
  },
  sortChanged : function(event){
    //dump("sort changed!-!-!-!\n");
  },
  onSelectView : function(type,event){
    //dump("type: "+type+"\n");
    var self = PlacesPreviewUtils;
    self.selected = type;
    if (type=="details"){
      if (!event || self.contentDeck.selectedIndex!=0){
        self.contentDeck.selectedIndex=0;
        var refresh = ThumbViewUtils.initiated;
        if (ThumbViewUtils.initiated)
          ThumbViewUtils.uninit();
        AlbumViewUtils.initDisplay(refresh);
      }
    } else if (type=="thumbnails"){
      if (!event || self.contentDeck.selectedIndex!=1){
        self.contentDeck.selectedIndex=1;
        if (AlbumView.initiated){
          AlbumView.uninit();
          ThumbViewUtils.initDisplay();
        }
        else{/* first load, give time for the contents to get set up */
          setTimeout(function(){ThumbViewUtils.initDisplay();},0);
        }
      }
    }
  },
  getViewRoot : function(aCurIndex, aRootIndex){
    var root;
    var curIndex = PlacesOrganizer._content.currentIndex;
    var rootIndex = -1;
    /*
    if (curIndex>-1 && PlacesOrganizer._content.view.isContainer(curIndex))
      rootIndex = curIndex;
    else if (curIndex>-1)
      rootIndex = PlacesOrganizer._content.view.getParentIndex(curIndex);
    if (rootIndex>-1)
      root = PlacesOrganizer._content.getResultView().nodeForTreeIndex(rootIndex);
    else
      root = PlacesOrganizer._content.getResultNode();
    */
    root = PlacesOrganizer._content.getResultNode();
    PlacesPreviewUtils.displayRoot = root;
    if (aCurIndex)
      aCurIndex.value = curIndex;
    if (aRootIndex)
      aRootIndex.value = rootIndex;
    //dump("root uri: "+root.uri+"\n");
    return root;
  },
  getImageForURL : function(aURL){
    var nopreview = "chrome://bookmarkpreviews/skin/nopreview.png";
    if (aURL){
      var file = PreviewServiceUtils.getPreviewFile(aURL,this.dir);
      if (file && file.exists() && file.isFile()){
        var ios = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
        var uri = ios.newFileURI(file);
        if (uri)
          return uri.spec;
      }
    }
    return nopreview;
  },
  getBookmarkImages : function(bkmks, aNode){
    if (!aNode)
      aNode = PlacesPreviewUtils.getViewRoot();
    var self = PlacesPreviewUtils;
    var view = PlacesOrganizer._content.getResultView();
    var idx = view.treeIndexForNode(aNode);
    if (idx == 4294967295) idx=-1;
    idx=-1;
    var isOpen = idx==-1 || view.isContainerOpen(idx);
    var rowCount = view.rowCount;
    if (!(rowCount==0 || idx>=rowCount)){    
      var level = view.getLevel(idx+1);
      for (var i = idx+1; i<rowCount; i++){
        var rowLevel = view.getLevel(i);
        if (rowLevel==level){
          var node = view.nodeForTreeIndex(i);
          var aType = node.type;
          if (aType!=Ci.nsINavHistoryResultNode.RESULT_TYPE_SEPARATOR){
            var url = node.uri;
            var name = node.title;
            var image = null;
            if (!(node instanceof Ci.nsINavHistoryContainerResultNode))
              image = PlacesPreviewUtils.getImageForURL(url);
            else
              image = "chrome://bookmarkpreviews/skin/folder.png";
            bkmks.push({image: image,
                        caption: name && name.length>0?name:url,
                        url: url,
                        node: node});
          }
        }
        else if (rowLevel<level){
          break;
        }
      }
    }
    /*
    if (bkmks.length == 0){
      bkmks.push({image: PlacesPreviewUtils.getImageForURL(),
                  caption: "",
                  url: null,
                  node: null});
    }*/
  },
  canDrop : function(index, orientation){},
  onDrop : function(row, orientation){},
  onToggleOpenState  : function(index){},
  onCycleHeader : function(column){},
  onCycleCell : function(row, column){},
  onSelectionChanged : function(event){
    //dump("onselectionchanged: "+event+"\n");
    if (PlacesPreviewUtils.utils.bkmksTree &&
        PlacesPreviewUtils.utils.bkmksTree.view!=PlacesPreviewUtils.utils.view){
      PlacesPreviewUtils.utils.view = PlacesPreviewUtils.utils.bkmksTree.view;
    }
    //dump("selection changed - "+AlbumView.updatedSelection+" - \n");
    if (!PlacesPreviewUtils.utils.updatedSelection){
      /*
      This listens to the detailed view and the folder view.
      Although the selection of the detailed view is updated in time when
      this is sent from the detailed view's tree, it isn't updated
      in time when it is sent from a selection change from the folder
      view
      */
      //AlbumViewUtils.refreshDisplay();
      setTimeout(function selectionChangedTimeout(){
        //dump("selection changed - "+PlacesPreviewUtils.utils.updatedSelection+" - \n");
        AlbumViewUtils.keepFocus = false
        PlacesPreviewUtils.utils.refreshDisplay();
        AlbumViewUtils.keepFocus = true;
      },5);
    }
  },
  onPerformAction : function(action){
    //dump("performaction\n");
  },
  onPerformActionOnRow : function(action, row){},
  onPerformActionOnCell : function(action, row, column){},
  observe: function(subject, topic, data) {
    //dump("observer: "+topic+"|"+subject+"\n");
    if (topic == "bookmarkpreviews:preview-updated"){
      /* subject = updated uri */
      //dump("update previews\n");
      this.utils.updatePreview(subject);
    }
    else{
      dump("topic did not match: "+topic+"|"+(topic == "bookmarkpreviews:preview-updated")+"\n");
    }
  },
  createPreviews : function(){
    var strbundleService = Components.classes ["@mozilla.org/intl/stringbundle;1"]
                        .getService(Components.interfaces.nsIStringBundleService);
    var bundle = strbundleService.createBundle("chrome://bookmarkpreviews/locale/strings.properties");
    var dialogQ = bundle.GetStringFromName("createpreviewsmessage");
    var dialogTitle = bundle.GetStringFromName("createpreviewsdialogtitle");
    var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                        .getService(Components.interfaces.nsIPromptService);
    var result = prompts.confirm(window, dialogTitle, dialogQ);
    if (result){
      var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                       .getService(Components.interfaces.nsIWindowMediator);
      var newWindow = wm.getMostRecentWindow("navigator:browser");
      if (newWindow)
        newWindow.PreviewService.generateAllPreviews();
      //XXX do something if there is no preview window...
    }
  }
}
var AlbumViewUtils = {
  displayView : null,
  placesTree : null,
  bkmksTree : null,
  view : null,
  canSelect : false,
  get displayRoot(){
    return PlacesPreviewUtils.displayRoot;
  },
  set displayRoot(val){
    PlacesPreviewUtils.displayRoot = val;
  },
  initDisplay : function(aForceRefresh){
    //dump("init albumview display\n");
    AlbumViewUtils.displayView = AlbumView;
    PlacesPreviewUtils.utils =AlbumViewUtils;
    AlbumViewUtils.placesTree = PlacesOrganizer._places;
    AlbumViewUtils.bkmksTree = PlacesOrganizer._content;
    AlbumViewUtils.view = AlbumViewUtils.bkmksTree.view;
    AlbumView.init();
    if (aForceRefresh)
      AlbumViewUtils.refreshDisplay(true);
    //setTimeout(function initDisplayTimeout(){
      //dump("init display\n");
      //AlbumViewUtils.refreshDisplay(true);
    //},0);
    //dump("finish init albumview display\n");
  },
  uninit : function(){
    this.initiated = false;
  },
  onResize : function(event){
    if (AlbumView.canRefresh){
      var orient = 1;
      if (event.target.id == "placesView")
        orient = 0;//Horizontal
      AlbumView.refresh(orient);
    }
  },
  _findNodeWithURI : function(root,uri){
    if (root.uri == uri) return root;
    for (var i=0;i<root.childCount;i++){
      var child = root.getChild(i);
      if (child.uri == uri)
        return child;
      if (child instanceof Components.interfaces.nsINavHistoryContainerResultNode){
        child=child.QueryInterface(Components.interfaces.nsINavHistoryContainerResultNode);
        if (child.containerOpen){
          if ((child = AlbumViewUtils._findNodeWithURI(child,uri)))
            return child;
        }
      }
    }
    return null;
  },
  refreshDisplay : function(aForceRefresh, what){
    //dump('refreshdisplay: '+aForceRefresh+" - ");
    var curIndexObj = new Object(), rootIndexObj = new Object();
    var previousRoot = AlbumViewUtils.displayRoot;
    var root = PlacesPreviewUtils.getViewRoot(curIndexObj, rootIndexObj);
    var curIndex = curIndexObj.value, rootIndex = rootIndexObj.value;

    if (aForceRefresh || root!=previousRoot){
      //dump("reload bookmarks\n");
      var bkmks=[];
      PlacesPreviewUtils.getBookmarkImages(bkmks,root);
      if (bkmks.length<50000){
        AlbumView.createImages(bkmks);
        AlbumView.current = AlbumView.target = 0;
      }
    }
    else{
      //dump(" scroll\n");
    }
    var nextTarget=AlbumViewUtils._getNextTarget(curIndex,rootIndex);
    //dump("aForceRefresh: "+aForceRefresh+"\n");
    //dump("curIndex: "+curIndex+" nextTarget: "+nextTarget+"\n");
    //nextTarget = 0;
    //AlbumViewUtils.displayRoot = root;
    AlbumView.keepFocus = false;
    AlbumView.refresh(-1);
    if (!(curIndex>-1 && AlbumViewUtils.view.isSeparator(curIndex)))
      AlbumView.glideTo(nextTarget*-150,false);

    this.initiated = true;
  },
  onResize : function(event){
    if (AlbumView.canRefresh){
      var orient = 1;
      if (event.target.id == "bookmark-views-splitter")
        orient = 0;//Horizontal
      AlbumView.refresh(orient);
    }
  },
  _getNextTarget : function(cIndex,rIndex){
    //dump("get_next_target c,r: "+cIndex+" ; "+rIndex+"\n");
    if (cIndex>rIndex+1){
      var max = AlbumViewUtils.view.rowCount, level = AlbumViewUtils.view.getLevel(cIndex);
      var nextTarget = 0;//rIndex==-1?-1:0;
      for (var i = rIndex+1;i<cIndex;i++){
        if (AlbumViewUtils.view.getLevel(i)==level &&
           !AlbumViewUtils.view.isSeparator(i))
          nextTarget++;
      }
      return nextTarget;
    }
    return 0;
  },
  updatePreview : function(aURI){
    //dump("update album preview: "+aURI.spec+"\n");
    aURI = aURI.QueryInterface(Components.interfaces.nsIURI);
    for (var i = 0; i < AlbumView.max; i++){
      var url = AlbumView.itemProps[i].url;
      if (url == aURI.spec){
        var imgsrc = PlacesPreviewUtils.getImageForURL(url);
        AlbumView.itemImages[i]=new Image();
        AlbumView.itemImages[i].onload = function(){
          AlbumView.imageLoaded(i);
        }
        AlbumView.itemImages[i].src = imgsrc;
        if (AlbumView.itemImages[i].complete){
          AlbumView.imageLoaded(i);
        }
        break;
      }
    }
  },
  selectIndex : function(node){
    if (AlbumViewUtils.keepFocus){
      AlbumViewUtils.updatedSelection = true;
      if (false && !(AlbumViewUtils.bkmksTree.hasSelection &&
          AlbumViewUtils.bkmksTree.hasSingleSelection &&
          AlbumViewUtils.bkmksTree.selectedNode == node)){
        AlbumViewUtils.view.selection.clearSelection();
        //AlbumViewUtils.bkmksTree.selectItems([node.itemId]);
        var view = AlbumViewUtils.bkmksTree.view.QueryInterface(Ci.nsINavHistoryResultTreeViewer);
        var index = view.treeIndexForNode(node);
        view.selection.select(index);
        AlbumViewUtils.bkmksTree.treeBoxObject.ensureRowIsVisible(index);

      }
      AlbumViewUtils.updatedSelection = false;
    }
    AlbumViewUtils.keepFocus = true;
  },
  performAction : function(node, event){
    if (PlacesUtils.nodeIsBookmark(node)){
      openUILink(node.uri,event);
    }
    else if (PlacesUtils.nodeIsContainer(node)){
      AlbumViewUtils.placesTree.selectItems([node.itemId]);
      //dump("perform action refresh\n");
      AlbumViewUtils.refreshDisplay();
    }
  }
}
var ThumbViewUtils = {
  displayView : null,
  placesTree : null,
  view : null,
  initDisplay : function(){
    //dump("init thumbview display\n");
    ThumbViewUtils.displayView = ThumbView;
    ThumbViewUtils.placesTree = PlacesOrganizer._places;
    PlacesPreviewUtils.utils  = ThumbViewUtils;
    document.getElementById('thumbsSize').removeAttribute("disabled");
    this.refreshDisplay(true);
    this.initiated = true;
    //dump("finish init thumbview display\n");
  },
  uninit : function(){
    this.initiated = false;
    document.getElementById('thumbsSize').setAttribute("disabled","true");
    /* select the current items in the content tree */
    var selectedNode = ThumbView.selectedNode;
    if (selectedNode)
      PlacesOrganizer._content.selectItems([selectedNode.itemId]);
  },
  refreshDisplay : function(aForceRefresh){
    //dump("refresh thumb display: "+aForceRefresh+"\n");
    this.initiated = true;
    var previousRoot = PlacesPreviewUtils.displayRoot;
    var root = PlacesPreviewUtils.getViewRoot();
    if (aForceRefresh || !previousRoot || previousRoot.itemId!=root.itemId){
      ThumbView.place = root.uri;
    } else{
      var bkmks = [];
      PlacesPreviewUtils.getBookmarkImages(bkmks);
      ThumbView.load(bkmks);
    }
  },
  onResize : function(event){
    ThumbView.onResize(event);
  },
  updatePreview : function(aURI){
    //dump("update preview: "+aURI+"\n");
    aURI = aURI.QueryInterface(Components.interfaces.nsIURI);
    ThumbView.updateThumbnail(aURI.spec);
  },
  updateNodeTitle : function(aNode, aPrevVal, aNewVal){
    ThumbView.updateNodeTitle(aNode,aPrevVal,aNewVal);
  },
  back : function(){PlacesOrganizer.back();},
  forward : function(){PlacesOrganizer.forward();},
  getBookmarkImages : PlacesPreviewUtils.getBookmarkImages,
  _getNextTarget : function(cIndex,rIndex){
    if (cIndex>rIndex+1){
      var max = ThumbViewUtils.view.rowCount, level = ThumbViewUtils.view.getLevel(cIndex);
      var nextTarget = 0;//rIndex==-1?-1:0;
      for (var i = rIndex+1;i<cIndex;i++){
        if (ThumbViewUtils.view.getLevel(i)==level &&
           !ThumbViewUtils.view.isSeparator(i))
          nextTarget++;
      }
      return nextTarget;
    }
    return 0;
  },
  selectIndex : function(node){
    ThumbViewUtils.updatedSelection = true;
    ThumbViewUtils.bkmksTree.selectItems([node.itemId]);
    //ThumbViewUtils.selectNode(ThumbViewUtils.bkmksTree,node);
    ThumbViewUtils.updatedSelection = false;
  },
  selectItem : function(item, event){
    ThumbViewUtils.updatedSelection = true;
    if (gEditItemOverlay.itemId != -1) {
      var focusedElement = document.commandDispatcher.focusedElement;
      if ((focusedElement instanceof HTMLInputElement ||
           focusedElement instanceof HTMLTextAreaElement) &&
          /^editBMPanel.*/.test(focusedElement.parentNode.parentNode.id))
        focusedElement.blur();
    }
    var contentTree = document.getElementById("placeContent");
    var detailsDeck = document.getElementById("detailsDeck");
    if (item) {
      detailsDeck.selectedIndex = 1;
      var selectedNode = item.node;
      if (selectedNode && selectedNode.itemId != -1) {
        //PlacesOrganizer._content.selectItems([selectedNode.itemId]);
        if (PlacesOrganizer._paneDisabled) {
          PlacesOrganizer._setDetailsFieldsDisabledState(false);
          PlacesOrganizer._paneDisabled = false;
        }
        gEditItemOverlay.initPanel(selectedNode.itemId,
                                   { hiddenRows: ["folderPicker"] });
        
        PlacesOrganizer._detectAndSetDetailsPaneMinimalState(selectedNode);
        ThumbViewUtils.updatedSelection = false;
        return;
      }
    }
    else {
      detailsDeck.selectedIndex = 0;
      var selectItemDesc = document.getElementById("selectItemDescription");
      var itemsCountLabel = document.getElementById("itemsCountText");
      var rowCount = ThumbView.bookmarks.length;
      if (rowCount == 0) {
        selectItemDesc.hidden = true;
        itemsCountLabel.value = PlacesUtils.getString("detailsPane.noItems");
      }
      else {
        selectItemDesc.hidden = false;
        if (rowCount == 1)
          itemsCountLabel.value = PlacesUtils.getString("detailsPane.oneItem");
        else {
          itemsCountLabel.value =
            PlacesUtils.getFormattedString("detailsPane.multipleItems",
                                           [rowCount]);
        }
      }
    }

    // Nothing to do if the pane was already disabled
    if (!PlacesOrganizer._paneDisabled) {
      gEditItemOverlay.uninitPanel();
      PlacesOrganizer._setDetailsFieldsDisabledState(true);
      PlacesOrganizer._paneDisabled = true;
    }
    ThumbViewUtils.updatedSelection = false;
  },
  performAction : function(item, event){
    var node = item.node;
    if (PlacesUtils.nodeIsBookmark(node)){
      openUILink(node.uri,event);
    }
    else if (PlacesUtils.nodeIsContainer(node)){
      //ThumbViewUtils.selectPlaceURI(PlacesOrganizer._places,node.uri);
      ThumbViewUtils.placesTree.selectItems([node.itemId]);
      //PlacesOrganizer.location = node.uri;
    }
  },
  removeItem : function(item, event){
    var node = item.node;
    PlacesUtils.bookmarks.removeItem(node.itemId);
    ThumbViewUtils.refreshDisplay();
  }
}
window.addEventListener('load',PlacesPreviewUtils.init,false);
window.addEventListener('unload',PlacesPreviewUtils.uninit,false);
